
from odoo import Command
from odoo.tests import tagged
from .common import HttpCaseWithUserPortalAndWirePayment


@tagged('post_install', '-at_install')
class TestCertification(HttpCaseWithUserPortalAndWirePayment):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()

        cls.odoo_certif_A = cls._create_test_certification("Odoo Certification A")
        cls.odoo_certif_B = cls._create_test_certification("Odoo Certification B")

    @classmethod
    def _create_test_certification(cls, name):
        certif_product = cls.env["product.product"].create({
            'name': name,
            'detailed_type': 'course',
            'is_published': True,
            'list_price': 245.0,
        })
        certif_survey = cls.env["survey.survey"].create({
            'title': name,
        })
        certif_channel = cls.env["slide.channel"].create({
            'name': name,
            'enroll': 'payment',
            'product_id': certif_product.id,
            'is_published': True,
            'channel_type': 'training',
            'tag_ids': [
                Command.set([cls.env.ref('openerp_portal.slide_channel_tag_certification').id])
            ],
        })
        certif_slide = cls.env["slide.slide"].create({
            'name': name,
            'slide_category': 'certification',
            'survey_id': certif_survey.id,
            'channel_id': certif_channel.id,
            'is_published': True,
        })
        return {
            'product': certif_product,
            'survey': certif_survey,
            'channel': certif_channel,
            'slide': certif_slide,
        }

    def test_start_certfication(self):
        """Test that user can start certification for SO not ordered from the website (via salesperson.)"""
        certif_order = self.env["sale.order"].create({
            'client_order_ref': 'M2023TESTCERTIF',
            'partner_id': self.partner_portal.id,
            'order_line': [
                Command.create({
                    'product_id': self.odoo_certif_A["product"].product_variant_id.id,
                    'product_uom_qty': 2,
                })
            ]
        })
        with self.assertLogs('odoo.addons.openerp_portal.models.sale_order') as cm:
            certif_order.action_confirm()
        self.assertIn(
            "odoo.addons.openerp_portal.models.sale_order:2 certif assigned to SaleOrder",
            cm.output[0]
        )
        self.assertEqual(certif_order.order_line.certif_channel_id, self.odoo_certif_A["channel"])
        self.assertEqual(certif_order.order_line.certif_available, 2)

        certif_invoice = certif_order._create_invoices()
        certif_invoice._post()
        self._register_payment(certif_invoice)

        self.start_tour('/my/orders', 'start_certification_tour', login='portal')

        self.assertEqual(certif_order.order_line.certif_available, 1)

    def test_start_certfication_change_version(self):
        """Test that user can change the certification version we want to take."""
        certif_order = self.env["sale.order"].create({
            'client_order_ref': 'M2023TESTCERTIF',
            'partner_id': self.partner_portal.id,
            'order_line': [
                Command.create({
                    'product_id': self.odoo_certif_A["product"].id,
                    'product_uom_qty': 2,
                }),
                Command.create({
                    'product_id': self.odoo_certif_B["product"].id,
                    'product_uom_qty': 3,
                })
            ]
        })
        certif_order_line_A = certif_order.order_line[0]
        certif_order_line_B = certif_order.order_line[1]

        with self.assertLogs('odoo.addons.openerp_portal.models.sale_order') as cm:
            certif_order.action_confirm()
        self.assertIn(
            "odoo.addons.openerp_portal.models.sale_order:2 certif assigned to SaleOrder",
            cm.output[0]
        )
        self.assertIn(
            "odoo.addons.openerp_portal.models.sale_order:3 certif assigned to SaleOrder",
            cm.output[1]
        )
        self.assertEqual(certif_order_line_A.certif_channel_id, self.odoo_certif_A["channel"])
        self.assertEqual(certif_order_line_A.certif_available, 2)
        self.assertEqual(certif_order_line_B.certif_channel_id, self.odoo_certif_B["channel"])
        self.assertEqual(certif_order_line_B.certif_available, 3)

        certif_invoice = certif_order._create_invoices()
        certif_invoice._post()
        self._register_payment(certif_invoice)

        self.start_tour('/my/orders', 'start_certification_tour_change_version', login='portal', timeout=3600)

        # tour will use the Manage Certification for the last line of the order
        # (so Odoo Certification B here), if use change the version of the
        # certification we want to take we should still decrease the certif.
        # available quantity, even if the certification channel does not match.
        self.assertEqual(certif_order_line_A.certif_available, 2)
        self.assertEqual(certif_order_line_B.certif_available, 2)  # 3 (ordered) - 1 (used)

    def test_self_buy_certification(self):
        """Test that a website user can buy a certification and is automatically enrolled."""
        self._setup_payment_transfer_for_test(set_done=True)

        self.start_tour('/slides', 'buy_certification_tour', login='portal')
